@@ -246,7 +246,7 @@ module Hledger.Cli.Balance (
 ) where
 
 import Data.List (intercalate)
-import Data.Maybe (fromMaybe, isJust)
+import Data.Maybe
 import Data.Monoid
 -- import Data.Text (Text)
 import qualified Data.Text as T
@@ -275,7 +275,7 @@ balancemode = (defCommandMode $ ["balance"] ++ aliases) { -- also accept but don
         "show historical ending balance in each period (includes postings before report start date)\n "
      ,flagNone ["tree"] (\opts -> setboolopt "tree" opts) "show accounts as a tree; amounts include subaccounts (default in simple reports)"
      ,flagNone ["flat"] (\opts -> setboolopt "flat" opts) "show accounts as a list; amounts exclude subaccounts except when account is depth-clipped (default in multicolumn reports)\n "
-     ,flagNone ["value","V"] (setboolopt "value") "convert amounts to current market value in their default valuation commodity"
+     ,flagNone ["value","V"] (setboolopt "value") "convert amounts to their market value on the report end date (using the most recent applicable market price, if any)"
      ,flagNone ["average","A"] (\opts -> setboolopt "average" opts) "show a row average column (in multicolumn reports)"
      ,flagNone ["row-total","T"] (\opts -> setboolopt "row-total" opts) "show a row total column (in multicolumn reports)"
      ,flagNone ["no-total","N"] (\opts -> setboolopt "no-total" opts) "omit the final total row"
@@ -300,7 +300,7 @@ balance opts@CliOpts{reportopts_=ropts} j = do
       let format   = outputFormatFromOpts opts
           interval = interval_ ropts
           baltype  = balancetype_ ropts
-          valuedate = fromMaybe d $ queryEndDate False $ queryFromOpts d ropts
+      mvaluedate <- reportEndDate j ropts
       -- shenanigans: use single/multiBalanceReport when we must,
       -- ie when there's a report interval, or --historical or -- cumulative.
       -- Otherwise prefer the older balanceReport since it can elide boring parents.
@@ -314,16 +314,16 @@ balance opts@CliOpts{reportopts_=ropts} j = do
                                | otherwise   = ropts{accountlistmode_=ALTree}
                     in singleBalanceReport ropts' (queryFromOpts d ropts) j
                 | otherwise = balanceReport ropts (queryFromOpts d ropts) j
-              convert | value_ ropts = balanceReportValue j valuedate
-                      | otherwise    = id
+              convert | value_ ropts = maybe id (balanceReportValue j) mvaluedate
+                      | otherwise = id
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
                 _     -> balanceReportAsText
           writeOutput opts $ render ropts $ convert report
         _ -> do
           let report = multiBalanceReport ropts (queryFromOpts d ropts) j
-              convert | value_ ropts = multiBalanceReportValue j valuedate
-                      | otherwise    = id
+              convert | value_ ropts = maybe id (multiBalanceReportValue j) mvaluedate
+                      | otherwise = id
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ multiBalanceReportAsCsv ropts r
                 _     -> case baltype of
