@@ -55,8 +55,12 @@ import Hledger.Cli.Accounts
 import Hledger.Cli.Balance
 import Hledger.Cli.Balancesheet
 import Hledger.Cli.Cashflow
+import Hledger.Cli.DocFiles
+import Hledger.Cli.Help
 import Hledger.Cli.Histogram
 import Hledger.Cli.Incomestatement
+import Hledger.Cli.Info
+import Hledger.Cli.Man
 import Hledger.Cli.Print
 import Hledger.Cli.Register
 import Hledger.Cli.Stats
@@ -66,7 +70,7 @@ import Hledger.Cli.Utils
 import Hledger.Cli.Version
 import Hledger.Data.Dates (getCurrentDay)
 import Hledger.Data.RawOptions (RawOpts, optserror)
-import Hledger.Reports.ReportOptions (dateSpanFromOpts, intervalFromOpts, queryFromOpts)
+import Hledger.Reports.ReportOptions (period_, interval_, queryFromOpts)
 import Hledger.Utils
 
 
@@ -98,6 +102,9 @@ mainmode addons = defMode {
                        cs -> [("\nAdd-on commands", map defAddonCommandMode cs)]
     -- modes in the unnamed group, shown first without a heading:
    ,groupUnnamed = [
+        helpmode
+       ,manmode
+       ,infomode
      ]
     -- modes handled but not shown
    ,groupHidden = [
@@ -157,26 +164,43 @@ argsToCliOpts args addons = do
 -- - move all required-argument help and input flags along with their values, space-separated or not
 -- - not confuse things further or cause misleading errors.
 moveFlagsAfterCommand :: [String] -> [String]
-moveFlagsAfterCommand args = move args
+moveFlagsAfterCommand args = moveArgs $ ensureDebugHasArg args
   where
-    move (f:a:as)           | isMovableNoArgFlag f           = (move $ a:as) ++ [f]
-    move (f:v:a:as)         | isMovableReqArgFlag f          = (move $ a:as) ++ [f,v]
-    move (fv:a:as)          | isMovableReqArgFlagAndValue fv = (move $ a:as) ++ [fv]
-    move ("--debug":v:a:as) | not (null v) && all isDigit v  = (move $ a:as) ++ ["--debug",v]
-    move ("--debug":a:as)                                    = (move $ a:as) ++ ["--debug"]
-    move (fv@('-':'-':'d':'e':'b':'u':'g':'=':_):a:as)       = (move $ a:as) ++ [fv]
-    move as = as
+    -- quickly! make sure --debug has a numeric argument, or this all goes to hell
+    ensureDebugHasArg as =
+      case break (=="--debug") as of
+       (bs,"--debug":c:cs) | null c || not (all isDigit c) -> bs++"--debug=1":c:cs
+       (bs,"--debug":[])                                   -> bs++"--debug=1":[]
+       _                                                   -> as
 
-    isMovableNoArgFlag a  = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` noargflagstomove
-    isMovableReqArgFlag a = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` reqargflagstomove
-    isMovableReqArgFlagAndValue ('-':'-':a:as) = case break (== '=') (a:as) of (f:fs,_) -> (f:fs) `elem` reqargflagstomove
-                                                                               _        -> False
-    isMovableReqArgFlagAndValue ('-':f:_:_) = [f] `elem` reqargflagstomove
-    isMovableReqArgFlagAndValue _ = False
+    -- -h ..., --version ...
+    moveArgs (f:a:as)   | isMovableNoArgFlag f                   = (moveArgs $ a:as) ++ [f]
+    -- -f FILE ..., --alias ALIAS ...
+    moveArgs (f:v:a:as) | isMovableReqArgFlag f, isValue v       = (moveArgs $ a:as) ++ [f,v]
+    -- -fFILE ..., --alias=ALIAS ...
+    moveArgs (fv:a:as)  | isMovableReqArgFlagAndValue fv         = (moveArgs $ a:as) ++ [fv]
+    -- -f(missing arg)
+    moveArgs (f:a:as)   | isMovableReqArgFlag f, not (isValue a) = (moveArgs $ a:as) ++ [f]
+    -- anything else
+    moveArgs as = as
 
-    noargflagstomove  = concatMap flagNames $ filter ((==FlagNone).flagInfo) flagstomove
-    reqargflagstomove = concatMap flagNames $ filter ((==FlagReq ).flagInfo) flagstomove
-    flagstomove = inputflags ++ helpflags
+isMovableNoArgFlag a  = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` noargflagstomove
+
+isMovableReqArgFlag a = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` reqargflagstomove
+
+isMovableReqArgFlagAndValue ('-':'-':a:as) = case break (== '=') (a:as) of (f:fs,_:_) -> (f:fs) `elem` reqargflagstomove
+                                                                           _          -> False
+isMovableReqArgFlagAndValue ('-':shortflag:_:_) = [shortflag] `elem` reqargflagstomove
+isMovableReqArgFlagAndValue _ = False
+
+isValue "-"     = True
+isValue ('-':_) = False
+isValue _       = True
+
+flagstomove = inputflags ++ helpflags
+noargflagstomove  = concatMap flagNames $ filter ((==FlagNone).flagInfo) flagstomove
+reqargflagstomove = -- filter (/= "debug") $
+                    concatMap flagNames $ filter ((==FlagReq ).flagInfo) flagstomove
 
 -- | Let's go.
 main :: IO ()
@@ -225,12 +249,22 @@ main = do
     isInternalCommand    = cmd `elem` builtinCommandNames -- not (null cmd) && not (cmd `elem` addons)
     isExternalCommand    = not (null cmd) && cmd `elem` addonPreciseNames -- probably
     isBadCommand         = not (null rawcmd) && null cmd
-    hasHelp args         = any (`elem` args) ["--help","-h","-?"]
     hasVersion           = ("--version" `elem`)
     hasDetailedVersion   = ("--version+" `elem`)
-    generalHelp          = putStr $ showModeHelp $ mainmode addonDisplayNames
+    printUsage           = putStr $ showModeUsage $ mainmode addonDisplayNames
     badCommandError      = error' ("command "++rawcmd++" is not recognized, run with no command to see a list") >> exitFailure
-    f `orShowHelp` mode  = if hasHelp args then putStr (showModeHelp mode) else f
+    hasShortHelpFlag args = any (`elem` args) ["-h"]
+    hasLongHelpFlag args = any (`elem` args) ["--help"]
+    hasManFlag args      = any (`elem` args) ["--man"]
+    hasInfoFlag args     = any (`elem` args) ["--info"]
+    hasSomeHelpFlag args = hasShortHelpFlag args || hasLongHelpFlag args || hasManFlag args || hasInfoFlag args
+    f `orShowHelp` mode
+      | hasShortHelpFlag args = putStr $ showModeUsage mode
+      | hasLongHelpFlag args  = printHelpForTopic t
+      | hasManFlag args       = runManForTopic t
+      | hasInfoFlag args      = runInfoForTopic t
+      | otherwise             = f
+      where t = topicForMode mode
   dbgIO "processed opts" opts
   dbgIO "command matched" cmd
   dbgIO "isNullCommand" isNullCommand
@@ -238,20 +272,23 @@ main = do
   dbgIO "isExternalCommand" isExternalCommand
   dbgIO "isBadCommand" isBadCommand
   d <- getCurrentDay
-  dbgIO "date span from opts" (dateSpanFromOpts d $ reportopts_ opts)
-  dbgIO "interval from opts" (intervalFromOpts $ reportopts_ opts)
+  dbgIO "period from opts" (period_ $ reportopts_ opts)
+  dbgIO "interval from opts" (interval_ $ reportopts_ opts)
   dbgIO "query from opts & args" (queryFromOpts d $ reportopts_ opts)
   let
     runHledgerCommand
-      -- high priority flags and situations. --help should be highest priority.
-      | hasHelp argsbeforecmd    = dbgIO "" "--help before command, showing general help" >> generalHelp
-      | not (hasHelp argsaftercmd) && (hasVersion argsbeforecmd || (hasVersion argsaftercmd && isInternalCommand))
+      -- high priority flags and situations. -h, then --help, then --info are highest priority.
+      | hasShortHelpFlag argsbeforecmd = dbgIO "" "-h before command, showing general usage" >> printUsage
+      | hasLongHelpFlag  argsbeforecmd = dbgIO "" "--help before command, showing general manual" >> printHelpForTopic (topicForMode $ mainmode addonDisplayNames)
+      | hasManFlag       argsbeforecmd = dbgIO "" "--man before command, showing general manual with man" >> runManForTopic (topicForMode $ mainmode addonDisplayNames)
+      | hasInfoFlag      argsbeforecmd = dbgIO "" "--info before command, showing general manual with info" >> runInfoForTopic (topicForMode $ mainmode addonDisplayNames)
+      | not (hasSomeHelpFlag argsaftercmd) && (hasVersion argsbeforecmd || (hasVersion argsaftercmd && isInternalCommand))
                                  = putStrLn prognameandversion
-      | not (hasHelp argsaftercmd) && (hasDetailedVersion argsbeforecmd || (hasDetailedVersion argsaftercmd && isInternalCommand))
+      | not (hasSomeHelpFlag argsaftercmd) && (hasDetailedVersion argsbeforecmd || (hasDetailedVersion argsaftercmd && isInternalCommand))
                                  = putStrLn prognameanddetailedversion
       -- \| (null externalcmd) && "binary-filename" `inRawOpts` rawopts = putStrLn $ binaryfilename progname
       -- \| "--browse-args" `elem` args     = System.Console.CmdArgs.Helper.execute "cmdargs-browser" mainmode' args >>= (putStr . show)
-      | isNullCommand            = dbgIO "" "no command, showing general help" >> generalHelp
+      | isNullCommand            = dbgIO "" "no command, showing general usage" >> printUsage
       | isBadCommand             = badCommandError
 
       -- internal commands
@@ -266,6 +303,9 @@ main = do
       | cmd == "register"        = withJournalDo opts register        `orShowHelp` registermode
       | cmd == "stats"           = withJournalDo opts stats           `orShowHelp` statsmode
       | cmd == "test"            = test' opts                         `orShowHelp` testmode
+      | cmd == "help"            = help' opts                         `orShowHelp` helpmode
+      | cmd == "man"             = man opts                           `orShowHelp` manmode
+      | cmd == "info"            = info' opts                         `orShowHelp` infomode
 
       -- an external command
       | isExternalCommand = do
